import socket
import struct
import os
from .encryption import Encryption

class Transport:
    def __init__(self):
        self.encryption = Encryption()
        self.shared_key = None

    def handshake(self, conn):
        """ Zabezpečená výměna klíčů pomocí ECDH a RSA podpisů """

        # 1️⃣ GENERACE NONCE pro ochranu proti replay attackům
        nonce = os.urandom(16)  # 128bitová náhodná výzva
        conn.sendall(nonce)

        # 2️⃣ ODESLÁNÍ veřejného klíče ECDH + PODPISU
        ec_public_bytes = self.encryption.ec_public_key.public_bytes(
            encoding=self.encryption.serialization.Encoding.X962,
            format=self.encryption.serialization.PublicFormat.UncompressedPoint
        )
        signature = self.encryption.sign_message(ec_public_bytes + nonce)

        conn.sendall(struct.pack("H", len(ec_public_bytes)) + ec_public_bytes)
        conn.sendall(struct.pack("H", len(signature)) + signature)

        # 3️⃣ PŘIJETÍ veřejného klíče + podpisu druhé strany
        key_length = struct.unpack("H", conn.recv(2))[0]
        peer_public_bytes = conn.recv(key_length)

        sig_length = struct.unpack("H", conn.recv(2))[0]
        peer_signature = conn.recv(sig_length)

        # 4️⃣ OVĚŘENÍ podpisu druhé strany
        if not self.encryption.verify_signature(peer_public_bytes + nonce, peer_signature, self.encryption.rsa_public_key):
            raise ValueError("Neplatný podpis! Možný MITM útok.")

        # 5️⃣ ODVOZENÍ sdíleného klíče
        self.shared_key = self.encryption.derive_shared_key(peer_public_bytes)

    def send_message(self, conn, message):
        """ Odeslání šifrované textové zprávy """
        if self.shared_key is None:
            raise ValueError("Neproběhla výměna klíčů!")

        encrypted_message = self.encryption.encrypt_message(message.encode(), self.shared_key)
        conn.sendall(struct.pack("I", len(encrypted_message)) + encrypted_message)

    def receive_message(self, conn):
        """ Přijetí šifrované textové zprávy """
        if self.shared_key is None:
            raise ValueError("Neproběhla výměna klíčů!")

        length = struct.unpack("I", conn.recv(4))[0]
        encrypted_message = conn.recv(length)
        return self.encryption.decrypt_message(encrypted_message, self.shared_key).decode()

    def send_file(self, conn, file_path):
        """ Odeslání šifrovaného souboru """
        if self.shared_key is None:
            raise ValueError("Neproběhla výměna klíčů!")

        with open(file_path, "rb") as file:
            file_data = file.read()

        encrypted_file = self.encryption.encrypt_message(file_data, self.shared_key)
        file_name = os.path.basename(file_path).encode()

        conn.sendall(struct.pack("H", len(file_name)) + file_name)  # Odeslat název souboru
        conn.sendall(struct.pack("I", len(encrypted_file)) + encrypted_file)  # Odeslat data

    def receive_file(self, conn, save_path):
        """ Přijetí šifrovaného souboru """
        if self.shared_key is None:
            raise ValueError("Neproběhla výměna klíčů!")

        file_name_length = struct.unpack("H", conn.recv(2))[0]
        file_name = conn.recv(file_name_length).decode()

        length = struct.unpack("I", conn.recv(4))[0]
        encrypted_file = conn.recv(length)

        decrypted_file = self.encryption.decrypt_message(encrypted_file, self.shared_key)

        full_path = os.path.join(save_path, file_name)
        with open(full_path, "wb") as file:
            file.write(decrypted_file)

        return full_path



import os
import struct
import hmac
import hashlib
from cryptography.hazmat.primitives.asymmetric import rsa, ec
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import padding

class Encryption:
    def __init__(self):
        # ECDH klíč pro výměnu
        self.ec_private_key = ec.generate_private_key(ec.SECP256R1())
        self.ec_public_key = self.ec_private_key.public_key()

        # RSA klíč pro digitální podpisy
        self.rsa_private_key = rsa.generate_private_key(
            public_exponent=65537, key_size=3072
        )
        self.rsa_public_key = self.rsa_private_key.public_key()

    def derive_shared_key(self, peer_public_bytes):
        """ Odvození sdíleného klíče pomocí ECDH """
        peer_public_key = ec.EllipticCurvePublicKey.from_encoded_point(ec.SECP256R1(), peer_public_bytes)
        shared_secret = self.ec_private_key.exchange(ec.ECDH(), peer_public_key)
        return HKDF(
            algorithm=hashes.SHA256(),
            length=32,
            salt=None,
            info=b'handshake data'
        ).derive(shared_secret)

    def encrypt(self, key, plaintext, associated_data=None):
        """ Šifrování pomocí AES-GCM """
        nonce = os.urandom(12)
        aesgcm = AESGCM(key)
        ciphertext = aesgcm.encrypt(nonce, plaintext, associated_data)
        return nonce + ciphertext

    def decrypt(self, key, ciphertext, associated_data=None):
        """ Dešifrování pomocí AES-GCM """
        nonce = ciphertext[:12]
        aesgcm = AESGCM(key)
        return aesgcm.decrypt(nonce, ciphertext[12:], associated_data)

    def sign(self, message):
        """ Podepsání zprávy pomocí RSA """
        signature = self.rsa_private_key.sign(
            message,
            padding.PSS(
                mgf=padding.MGF1(hashes.SHA256()),
                salt_length=padding.PSS.MAX_LENGTH
            ),
            hashes.SHA256()
        )
        return signature

    def verify(self, message, signature):
        """ Ověření podpisu pomocí RSA """
        try:
            self.rsa_public_key.verify(
                signature,
                message,
                padding.PSS(
                    mgf=padding.MGF1(hashes.SHA256()),
                    salt_length=padding.PSS.MAX_LENGTH
                ),
                hashes.SHA256()
            )
            return True
        except Exception:
            return False

    def get_ec_public_bytes(self):
        """ Získání veřejného klíče v X9.62 formátu """
        return self.ec_public_key.public_bytes(
            encoding=serialization.Encoding.X962,
            format=serialization.PublicFormat.UncompressedPoint
        )



from setuptools import setup, find_packages

setup(
    name="emproto",
    version="0.4.0",
    author="Alfisene Keita",
    description="Bezpečný přenos zpráv a souborů přes šifrování",
    url="https://github.com/certikpolik3/EM-Proto",
    packages=find_packages(),
    install_requires=[
        "cryptography"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
